﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Operators.LCS;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList {
  [Item("BestTrainingDecisionListAnalyzer", "Description missing")]
  [StorableClass]
  public class BestTrainingDecisionListAnalyzer : SingleSuccessorOperator, IAnalyzer {
    private const string BestTrainingSolutionName = "Best Training Solution";
    private const string TrainingBestSolutionQualityParameterName = "Best training solution quality";

    public bool EnabledByDefault {
      get { return true; }
    }

    #region Parameter Properties
    public ValueLookupParameter<ResultCollection> ResultsParameter {
      get { return (ValueLookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    public ScopeTreeLookupParameter<IGAssistIndividual> IndividualParameter {
      get { return (ScopeTreeLookupParameter<IGAssistIndividual>)Parameters["Individual"]; }
    }
    public LookupParameter<IDecisionListClassificationProblemData> ProblemDataParameter {
      get { return (LookupParameter<IDecisionListClassificationProblemData>)Parameters["ProblemData"]; }
    }
    public IScopeTreeLookupParameter<DoubleValue> QualityParameter {
      get { return (IScopeTreeLookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<BoolValue> MaximizationParameter {
      get { return (ILookupParameter<BoolValue>)Parameters["Maximization"]; }
    }
    #endregion

    public ResultCollection Results { get { return ResultsParameter.ActualValue; } }

    [StorableConstructor]
    protected BestTrainingDecisionListAnalyzer(bool deserializing) : base(deserializing) { }
    protected BestTrainingDecisionListAnalyzer(BestTrainingDecisionListAnalyzer original, Cloner cloner)
      : base(original, cloner) {
    }
    public BestTrainingDecisionListAnalyzer()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<IGAssistIndividual>("Individual", ""));
      Parameters.Add(new ValueLookupParameter<ResultCollection>("Results", "The result collection where the solution should be stored."));
      Parameters.Add(new LookupParameter<IDecisionListClassificationProblemData>("ProblemData", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Quality", "The qualities of the trees that should be analyzed."));
      Parameters.Add(new LookupParameter<BoolValue>("Maximization", "The direction of optimization."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new BestTrainingDecisionListAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      ItemArray<IGAssistIndividual> individuals = IndividualParameter.ActualValue;
      ItemArray<DoubleValue> qualities = QualityParameter.ActualValue;
      bool maximization = MaximizationParameter.ActualValue.Value;
      IDecisionListClassificationProblemData problemData = ProblemDataParameter.ActualValue;

      DoubleValue value = maximization ? qualities.MaxItems(x => x.Value).First() : qualities.MinItems(x => x.Value).First();

      int index = qualities.IndexOf(value);

      if (!Results.ContainsKey(BestTrainingSolutionName)) {
        Results.Add(new Result(TrainingBestSolutionQualityParameterName, new DoubleValue(value.Value)));
        Results.Add(new Result(BestTrainingSolutionName, new DecisionListSolution((DecisionList)individuals[index], problemData)));
      } else {
        double oldBestQuality = ((DoubleValue)Results[TrainingBestSolutionQualityParameterName].Value).Value;
        if (maximization && oldBestQuality < value.Value || !maximization && oldBestQuality > value.Value) {
          Results[BestTrainingSolutionName].Value = new DecisionListSolution((DecisionList)individuals[index], problemData);
          Results[TrainingBestSolutionQualityParameterName].Value = value;
        }
      }

      return base.Apply();
    }
  }
}
